model PI_Pump_Full_Sim
  extends NetworkSimulation;
  import SI = Modelica.SIunits;
  PI_controller PI_GEM_Gouda(measured = BG_Gouda.HQ.H, setpoint = RBP_H_STREEF);
  PI_controller PI_GEM_Katwijk(measured = BG_Katwijk.HQ.H, setpoint = RBP_H_STREEF);
  PI_controller PI_GEM_Spaarndam(measured = BG_Spaarndam.HQ.H, setpoint = RBP_H_STREEF);
  PI_controller PI_GEM_Halfweg(measured = BG_Halfweg.HQ.H, setpoint = RBP_H_STREEF);
  // This is a more "tuned" implementation, but it did not improve convergence at theta=1.0.
  // PI_controller PI_GEM_Gouda(measured = BG_Gouda.HQ.H, setpoint = RBP_H_STREEF, Kp = 30.0);
  // PI_controller PI_GEM_Katwijk(measured = BG_Katwijk.HQ.H, setpoint = RBP_H_STREEF, Kp = 120.0);
  // PI_controller PI_GEM_Spaarndam(measured = BG_Spaarndam.HQ.H, setpoint = RBP_H_STREEF, Kp = 240.0);
  // PI_controller PI_GEM_Halfweg(measured = BG_Halfweg.HQ.H, setpoint = RBP_H_STREEF, Kp = 240.0);
  output SI.VolumeFlowRate GEM_Gouda_Q = GEM_Gouda.Q;
  output SI.VolumeFlowRate GEM_Gouda_inflow_Q = GEM_Gouda_inflow.Q;
  output SI.VolumeFlowRate GEM_Katwijk_Q = GEM_Katwijk.Q;
  output SI.VolumeFlowRate GEM_Spaarndam_Q = GEM_Spaarndam.Q;
  output SI.VolumeFlowRate GEM_Halfweg_Q = GEM_Halfweg.Q;
  output SI.VolumeFlowRate GEM_N3MP_Q = GEM_N3MP.Q;
  output SI.VolumeFlowRate GEM_Haarlemmermeer_Q = GEM_Haarlemmermeer.Q;

  input SI.VolumeFlowRate GEM_Gouda_Qmin;
  input SI.VolumeFlowRate GEM_Gouda_Qmax;
  input SI.VolumeFlowRate GEM_Katwijk_Qmin;
  input SI.VolumeFlowRate GEM_Katwijk_Qmax;
  input SI.VolumeFlowRate GEM_Spaarndam_Qmin;
  input SI.VolumeFlowRate GEM_Spaarndam_Qmax;
  input SI.VolumeFlowRate GEM_Halfweg_Qmin;
  input SI.VolumeFlowRate GEM_Halfweg_Qmax;

  SI.VolumeFlowRate GEM_Gouda_Q_expected;
  SI.VolumeFlowRate GEM_Katwijk_Q_expected;
  SI.VolumeFlowRate GEM_Spaarndam_Q_expected;
  SI.VolumeFlowRate GEM_Halfweg_Q_expected;

  input SI.VolumeFlowRate GEM_Gouda_Q_actual = GEM_Gouda.Q;
  input SI.VolumeFlowRate GEM_Gouda_inflow_Q_actual = GEM_Gouda_inflow.Q;
  input SI.VolumeFlowRate GEM_Katwijk_Q_actual = GEM_Katwijk.Q;
  input SI.VolumeFlowRate GEM_Spaarndam_Q_actual = GEM_Spaarndam.Q;
  input SI.VolumeFlowRate GEM_Halfweg_Q_actual = GEM_Halfweg.Q;
  input SI.VolumeFlowRate GEM_N3MP_Q_actual = GEM_N3MP.Q;
  input SI.VolumeFlowRate GEM_Haarlemmermeer_Q_actual = GEM_Haarlemmermeer.Q;

  parameter SI.Position RBP_H_STREEF;

equation
  // Note: min and max timeseries values are assumed reversed due to FEWS shenanigans
  GEM_Gouda_Q_expected = max(min(PI_GEM_Gouda.control, GEM_Gouda_Qmin), GEM_Gouda_Qmax);
  GEM_Katwijk_Q_expected = max(min(PI_GEM_Katwijk.control, GEM_Katwijk_Qmin), GEM_Katwijk_Qmax);
  GEM_Spaarndam_Q_expected = max(min(PI_GEM_Spaarndam.control, GEM_Spaarndam_Qmin), GEM_Spaarndam_Qmax);
  GEM_Halfweg_Q_expected = max(min(PI_GEM_Halfweg.control, GEM_Halfweg_Qmin), GEM_Halfweg_Qmax);
end PI_Pump_Full_Sim;
