import csv
import logging
import sys
import rtctools.data.pi as pi

filenamecsv = r"Regimes_Rijnland.csv"
filenamegoalsxml = r"../input/rtcParameterConfig_Goals.xml"
filenameboezemxml = r"../input/rtcParameterConfig_Boezem.xml"
filenameknopenxml = r"../input/rtcParameterConfig_Knopen.xml"

# Set up logging
logger = logging.getLogger("rtctools")

# Add stream handler if it does not already exist.
if not any((isinstance(h, logging.StreamHandler) for h in logger.handlers)):
    handler = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

# Add pi.DiagHandler, but only if it does not already exist.
if not any((isinstance(h, pi.DiagHandler) for h in logger.handlers)):
    handler = pi.DiagHandler(output_folder)
    logger.addHandler(handler)

# Set initial log level to info
logger.setLevel(logging.INFO)

from xml.etree import ElementTree as et
logger.info("Regime runner: Scenario selection")
et.register_namespace('pi',"http://www.wldelft.nl/fews/PI")
goaltree = et.parse(filenamegoalsxml)
boezemtree = et.parse(filenameboezemxml)
knopentree = et.parse(filenameknopenxml)
config_goalparameters = goaltree.findall('.//{http://www.wldelft.nl/fews/PI}parameter')
config_boezem = boezemtree.findall('.//{http://www.wldelft.nl/fews/PI}parameter')
config_knopen = knopentree.findall('.//{http://www.wldelft.nl/fews/PI}parameter')

def loadscenario(scenario):
    with open(filenamecsv) as f:
        # csv_dialect = csv.Sniffer().sniff(f.readline(), [',',';'])
        # reader = csv.DictReader(f, dialect=csv_dialect)
        reader = csv.DictReader(f)
        try:
            for row in reader:
                if str(row['Scenarionummer'])==str(int(float(scenario))):
                    logger.info("The scenario timeseries is set to scenario %s", scenario)
                    logger.info("Setting goals in rtcParameterConfig_Goals.xml.")
                    for param in config_goalparameters:
                        if param.get('id') == 'use_H_range_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_RBP_marges'] == "1").lower()
                            logger.info("Setting H_BOVEN, H_STREEF en H_ONDER in rtcParameterConfig_Boezem.xml.")
                            for p in config_boezem:
                                if p.get('id') == 'H_BOVEN':
                                    print(p.text)
                                    p.find('{http://www.wldelft.nl/fews/PI}dblValue').text = str(row['RBP_boven'])
                                elif p.get('id') == 'H_STREEF':
                                    p.find('{http://www.wldelft.nl/fews/PI}dblValue').text = str(row['RBP_streef'])
                                elif p.get('id') == 'H_ONDER':
                                    p.find('{http://www.wldelft.nl/fews/PI}dblValue').text = str(row['RBP_onder'])
                        elif param.get('id') == 'use_pump_Hdown_range_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_gemaal_marges'] == "1").lower()
                        elif param.get('id') == 'use_katwijk_tide_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doelen_Katwijk_getij'] == "1").lower()
                        elif param.get('id') == 'use_Cl_flushing_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_doorspoelen'] == "1").lower()
                        elif param.get('id') == 'use_Cl_range_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_chloride_marges'] == "1").lower()
                        elif param.get('id') == 'use_Q_emergency_pump_minimize_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_inzet_noodbergingen'] == "1").lower()
                        elif param.get('id') == 'use_Q_pump_req_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_opgelegde_inzet_boezemgemalen'] == "1").lower()
                        elif param.get('id') == 'use_cost_pump_minimize_goal_var':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_minimaliseer_pompkosten'] == "1").lower()
                        elif param.get('id') == 'prioritize_between_pumps':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_inzetvolgorde_boezemgemalen'] == "1").lower()
                        elif param.get('id') == 'use_H_setpoint_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_RBP_streefpeil'] == "1").lower()
                        elif param.get('id') == 'use_Hnode_range_goal':
                            param.find('{http://www.wldelft.nl/fews/PI}boolValue').text = str(row['Doel_sturen_op_knoop'] == "1").lower()
                            #logger.info("Setting H_MIN, H_MAX in rtcParameterConfig_Knopen.xml.")
                            #for p in config_knopen:
                            #    logger.info(p)
                            #    if p.get('id') == 'H_MAX':
                            #        print(p.text)
                            #        p.find('{http://www.wldelft.nl/fews/PI}dblValue').text = str(row['Sloten_Hmax'])
                            #    elif p.get('id') == 'H_MIN':
                            #        p.find('{http://www.wldelft.nl/fews/PI}dblValue').text = str(row['Sloten_Hmin'])

                goaltree.write(open(filenamegoalsxml, 'wb'), xml_declaration=True, encoding="UTF-8", method="xml")
                boezemtree.write(open(filenameboezemxml, 'wb'), xml_declaration=True, encoding="UTF-8", method="xml")

        except csv.Error as e:
            sys.exit('file {}, line {}: {}'.format(filenamecsv, reader.line_num, e))
